@@ -93,6 +93,7 @@ static void vo_chain_reset_state(struct vo_chain *vo_c)
     vo_seek_reset(vo_c->vo);
     vo_c->underrun = false;
     vo_c->underrun_signaled = false;
+    vo_forget_frames(vo_c->vo);
 }
 
 void reset_video_state(struct MPContext *mpctx)
@@ -559,7 +560,13 @@ static int video_output_image(struct MPContext *mpctx, bool *logical_eof)
         if (mpctx->saved_frame)
             add_new_frame(mpctx, mpctx->saved_frame);
         mpctx->saved_frame = NULL;
-        *logical_eof = true;
+        if (!have_new_frame(mpctx, true)) {
+            if (mpctx->video_out) {
+                vo_wait_frame(mpctx->video_out);
+                vo_forget_frames(mpctx->video_out);
+            }
+            *logical_eof = true;
+        }
     }
 
     return have_new_frame(mpctx, r <= 0) ? VD_NEW_FRAME : r;
