@@ -1500,7 +1500,7 @@ static size_t vbuf_upload(struct ra *ra, void *data, size_t size)
     return offset;
 }
 
-static const char cache_magic[4] = "RD11";
+static const char cache_magic[4] MP_NONSTRING = "RD11";
 static const uint32_t cache_version = 4;
 
 struct cache_header {
@@ -2256,7 +2256,7 @@ static void destroy(struct ra *ra)
     talloc_free(ra);
 }
 
-static struct ra_fns ra_fns_d3d11 = {
+static const struct ra_fns ra_fns_d3d11 = {
     .destroy            = destroy,
     .tex_create         = tex_create,
     .tex_destroy        = tex_destroy,
